==> rd2020-element-highlight-1.8.6.tgz: package/src/index.js <==
import invariant from 'invariant';
import addEvent from '@thinkeloquent/rd2020-dom/src/element/addEvent';
import addClass from '@thinkeloquent/rd2020-dom/src/element/addClass';
import removeClass from '@thinkeloquent/rd2020-dom/src/element/removeClass';
import getTargetFromEvent from '@thinkeloquent/rd2020-dom/src/element/getTargetFromEvent';
import { findOneWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/findOne';
import { findWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/find';
import getTagName from '@thinkeloquent/rd2020-dom/src/element/getTagName';
import { getEditableTags } from '@thinkeloquent/rd2020-element-meta/src';

import getNodeTypeFromElement, { DOCUMENT_NODE } from '@thinkeloquent/rd2020-dom/src/element/getNodeTypeFromElement';
import stylesheet from './index.css';

let DEBUG = false;
export const setDebug = (boolean) => DEBUG = Boolean(boolean);

export const highlightClassName = `root-${process.env.BUILD_ID}`;
export const highlightSelectedClassName = `selected-${process.env.BUILD_ID}`;
export const headStyleTagsId = `style-${process.env.BUILD_ID}`;
export { getEditableTags }

export const fromDocument = (document) => ({
  body: document.body,
  head: document.head,
  document,
});

export const fromIframe = (document) => ({
  body: document.body,
  head: document.head,
  document,
});

export const logger = (...args) => {
  if(!DEBUG) return;
  console.log('ELEMENT_HIGHLIGHT', ...args);
};

class ElementHighlight {
  constructor(element) {
    const isDocument = getNodeTypeFromElement(element).nodeType === DOCUMENT_NODE;
    const {
      body,
      head,
      document,
    } = isDocument ? fromDocument(element) : fromIframe(element);
    
    this.dom = {
      body,
      head,
      document,
    }

    this.previousHoveredElement = null;
    this.previousSelectedElement = null;
    this.disposeOf = [];

    this.classes = {
      ignoreElement: stylesheet.locals.highlight,
      highlight: stylesheet.locals.highlight,
      highlightSelected: stylesheet.locals.highlightSelected,
    };

    this.insertStyleSheet();
  }

  useStyleSheet() {
    stylesheet.use();
  }

  insertStyleSheet() {
    this.stylesheet = document.createElement('style');
    this.stylesheet.id = headStyleTagsId;
    this.stylesheet.innerHTML = stylesheet.toString();
    this.dom.head.insertAdjacentHTML('beforeend', this.stylesheet.outerHTML);
  }

  removeClassName(className) {
    let currentlySelected = findWithDocument(this.dom.document, `.${className}`);

    if (currentlySelected && currentlySelected.length) {
        removeClass(currentlySelected, className);
    }
    return currentlySelected;
  }

  getSelected() {
    return findOneWithDocument(this.dom.document, this.classes.highlightSelected);
  }

  ignoreElement(query) {
    const element = findOneWithDocument(this.dom.document, query);

    if(!element) {
      logger(`ignore element not found: ${query}`);
      return;
    }

    element.setAttribute('hightlight-ignore-element', "true");

    addEvent(element, 'click', (event) => {
      logger(`ignore element ${query}`);
      event.preventDefault();
      event.stopPropagation();
    });
  }

  disposal() {
    for(let index in this.disposeOf) {
      this.disposeOf[index]();
    }
  }

  cleanup() {
    this.removeHighlight()
    this.disposal();
  }

  removeHighlight() {
    this.previousHoveredElement = null;
    this.previousSelectedElement = null;
    this.removeClassName(this.classes.highlight);
    this.removeClassName(this.classes.highlightSelected);
  }

  mouseover({
    editableTags
  }) {
    this.disposeOf.push(
      addEvent(this.dom.body, 'mouseover', (event) => {
        const target = getTargetFromEvent(event);
        const tagName = getTagName(target);
        console.log(tagName, editableTags.indexOf(tagName) === -1, this.classes.highlight);

        if(editableTags.indexOf(tagName) === -1) {
          logger(`ignore element mouseover: ${tagName}`);
          return;
        }

        target.contentEditable = "true";
        addClass(target, this.classes.highlight);
        this.previousHoveredElement = target;
      })
    )
  }

  mouseout() {
    this.disposeOf.push(
      addEvent(this.dom.body, 'mouseout', (event) => {
        const element = event ? event.target : this.previousHoveredElement;
        if (element) removeClass(element, this.classes.highlight);
      })
    )
  }

  click(options) {
    const {
      onIgnore,
      isContextTemplate,
      editableTags,
      onReSelect,
    } = options;
    this.disposeOf.push(
      addEvent(this.dom.body, 'click', (event) => {
        const target = getTargetFromEvent(event);
        const tagName = getTagName(target);
        
        if(isContextTemplate(target) || this.previousSelectedElement === target) {
          if(typeof onReSelect === 'function') onReSelect(this.previousSelectedElement);
          logger(`ignore context template click: ${tagName}`);
          return;
        }

        if(editableTags.indexOf(tagName) === -1) {
          logger(`ignore element click: ${tagName}`);
          if(typeof onIgnore === 'function') onIgnore(target, this.previousSelectedElement);
          return;
        }

        event.preventDefault();
        event.stopPropagation();
        
        this.selectElement(target, options);
      })
    )
  }

  selectElement(target, {
    onSelect,
    onRemove,
  }) {
    if(this.previousSelectedElement) {
      if(typeof onRemove === 'function') onRemove(target, this.previousSelectedElement);
      removeClass(this.previousSelectedElement, this.classes.highlightSelected);
    }

    addClass(target, this.classes.highlightSelected);

    if(typeof onSelect === 'function') onSelect(target, this.previousSelectedElement);

    this.previousSelectedElement = target;
  }

  initialize(argOptions = {}) {
    const options = Object.assign({}, argOptions);
    invariant(options.onReSelect, 'onReSelect function is missing');
    invariant(options.onIgnore, 'onIgnore function is missing');
    invariant(options.onRemove, 'onRemove function is missing');
    invariant(options.onSelect, 'onSelect function is missing');
    invariant(Array.isArray(options.editableTags), 'editableTags array is missing');
    invariant(options.isContextTemplate, 'isContextTemplate function is missing');
    this.mouseover(options);
    this.mouseout(options);
    this.click(options);
  }

  getData() {
    return {}
  }

}

export default ElementHighlight;

==> rd2020-element-highlight-1.8.7.tgz: package/src/index.js <==
import invariant from 'invariant';
import addEvent from '@thinkeloquent/rd2020-dom/src/element/addEvent';
import addClass from '@thinkeloquent/rd2020-dom/src/element/addClass';
import removeClass from '@thinkeloquent/rd2020-dom/src/element/removeClass';
import getTargetFromEvent from '@thinkeloquent/rd2020-dom/src/element/getTargetFromEvent';
import { findOneWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/findOne';
import { findWithDocument } from '@thinkeloquent/rd2020-dom/src/selector/find';
import getTagName from '@thinkeloquent/rd2020-dom/src/element/getTagName';
import { getEditableTags } from '@thinkeloquent/rd2020-element-meta/src';

import getNodeTypeFromElement, { DOCUMENT_NODE } from '@thinkeloquent/rd2020-dom/src/element/getNodeTypeFromElement';
import stylesheet from './index.css';

let DEBUG = false;
export const setDebug = (boolean) => DEBUG = Boolean(boolean);

export const highlightClassName = `root-${process.env.BUILD_ID}`;
export const highlightSelectedClassName = `selected-${process.env.BUILD_ID}`;
export const headStyleTagsId = `style-${process.env.BUILD_ID}`;
export { getEditableTags }

export const fromDocument = (document) => ({
  body: document.body,
  head: document.head,
  document,
});

export const fromIframe = (document) => ({
  body: document.body,
  head: document.head,
  document,
});

export const logger = (...args) => {
  if(!DEBUG) return;
  console.log('ELEMENT_HIGHLIGHT', ...args);
};

class ElementHighlight {
  constructor(element) {
    const isDocument = getNodeTypeFromElement(element).nodeType === DOCUMENT_NODE;
    const {
      body,
      head,
      document,
    } = isDocument ? fromDocument(element) : fromIframe(element);
    
    this.dom = {
      body,
      head,
      document,
    }

    this.previousHoveredElement = null;
    this.previousSelectedElement = null;
    this.disposeOf = [];

    this.classes = {
      ignoreElement: stylesheet.locals.highlight,
      highlight: stylesheet.locals.highlight,
      highlightSelected: stylesheet.locals.highlightSelected,
    };

    this.insertStyleSheet();
  }

  useStyleSheet() {
    if(stylesheet.use) stylesheet.use();
  }

  insertStyleSheet() {
    this.stylesheet = document.createElement('style');
    this.stylesheet.id = headStyleTagsId;
    this.stylesheet.innerHTML = stylesheet.toString();
    this.dom.head.insertAdjacentHTML('beforeend', this.stylesheet.outerHTML);
  }

  removeClassName(className) {
    let currentlySelected = findWithDocument(this.dom.document, `.${className}`);

    if (currentlySelected && currentlySelected.length) {
        removeClass(currentlySelected, className);
    }
    return currentlySelected;
  }

  getSelected() {
    return findOneWithDocument(this.dom.document, this.classes.highlightSelected);
  }

  ignoreElement(query) {
    const element = findOneWithDocument(this.dom.document, query);

    if(!element) {
      logger(`ignore element not found: ${query}`);
      return;
    }

    element.setAttribute('hightlight-ignore-element', "true");

    addEvent(element, 'click', (event) => {
      logger(`ignore element ${query}`);
      event.preventDefault();
      event.stopPropagation();
    });
  }

  disposal() {
    for(let index in this.disposeOf) {
      this.disposeOf[index]();
    }
  }

  cleanup() {
    this.removeHighlight()
    this.disposal();
    if(stylesheet.unuse) stylesheet.unuse();
  }

  removeHighlight() {
    this.previousHoveredElement = null;
    this.previousSelectedElement = null;
    this.removeClassName(this.classes.highlight);
    this.removeClassName(this.classes.highlightSelected);
  }

  mouseover({
    editableTags
  }) {
    this.disposeOf.push(
      addEvent(this.dom.body, 'mouseover', (event) => {
        const target = getTargetFromEvent(event);
        const tagName = getTagName(target);

        if(editableTags.indexOf(tagName) === -1) {
          logger(`ignore element mouseover: ${tagName}`);
          return;
        }

        //target.contentEditable = "true";
        addClass(target, this.classes.highlight);
        this.previousHoveredElement = target;
      })
    )
  }

  mouseout() {
    this.disposeOf.push(
      addEvent(this.dom.body, 'mouseout', (event) => {
        const element = event ? event.target : this.previousHoveredElement;
        if (element) removeClass(element, this.classes.highlight);
      })
    )
  }

  click(options) {
    const {
      onIgnore,
      isContextTemplate,
      editableTags,
      onReSelect,
    } = options;
    this.disposeOf.push(
      addEvent(this.dom.body, 'click', (event) => {
        const target = getTargetFromEvent(event);
        const tagName = getTagName(target);
        
        if(isContextTemplate(target) || this.previousSelectedElement === target) {
          if(typeof onReSelect === 'function') onReSelect(this.previousSelectedElement);
          logger(`ignore context template click: ${tagName}`);
          return;
        }

        if(editableTags.indexOf(tagName) === -1) {
          logger(`ignore element click: ${tagName}`);
          if(typeof onIgnore === 'function') onIgnore(target, this.previousSelectedElement);
          return;
        }

        event.preventDefault();
        event.stopPropagation();
        
        this.selectElement(target, options);
      })
    )
  }

  selectElement(target, {
    onSelect,
    onRemove,
  }) {
    if(this.previousSelectedElement) {
      if(typeof onRemove === 'function') onRemove(target, this.previousSelectedElement);
      removeClass(this.previousSelectedElement, this.classes.highlightSelected);
    }

    addClass(target, this.classes.highlightSelected);

    if(typeof onSelect === 'function') onSelect(target, this.previousSelectedElement);

    this.previousSelectedElement = target;
  }

  initialize(argOptions = {}) {
    const options = Object.assign({}, argOptions);
    invariant(options.onReSelect, 'onReSelect function is missing');
    invariant(options.onIgnore, 'onIgnore function is missing');
    invariant(options.onRemove, 'onRemove function is missing');
    invariant(options.onSelect, 'onSelect function is missing');
    invariant(Array.isArray(options.editableTags), 'editableTags array is missing');
    invariant(options.isContextTemplate, 'isContextTemplate function is missing');
    this.mouseover(options);
    this.mouseout(options);
    this.click(options);
  }

  getData() {
    return {}
  }

}

export default ElementHighlight;
